/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * MoleculeByOrderAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "molecule.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "Descriptors/MoleculeOrderDescriptor.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

#include "MoleculeByOrderAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "MoleculeByOrderAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */

Action::state_ptr SelectionMoleculeByOrderAction::performCall() {
  molecule *mol = World::getInstance().getMolecule(MoleculeByOrder(params.molindex.get()));

  if (mol != NULL) {
    if (!World::getInstance().isSelected(mol)) {
      LOG(1, "Selecting molecule " << mol->name);
      World::getInstance().selectMolecule(mol);
      LOG(0, World::getInstance().countSelectedMolecules() << " molecules selected.");
      return Action::state_ptr(new SelectionMoleculeByOrderState(mol, params));
    } else {
      return Action::success;
    }
  } else {
    return Action::failure;
  }
}

Action::state_ptr SelectionMoleculeByOrderAction::performUndo(Action::state_ptr _state) {
  SelectionMoleculeByOrderState *state = assert_cast<SelectionMoleculeByOrderState*>(_state.get());

  World::getInstance().unselectMolecule(state->mol);
  return Action::state_ptr(_state);
}

Action::state_ptr SelectionMoleculeByOrderAction::performRedo(Action::state_ptr _state){
  SelectionMoleculeByOrderState *state = assert_cast<SelectionMoleculeByOrderState*>(_state.get());

  World::getInstance().selectMolecule(state->mol);
  return Action::state_ptr(_state);
}

bool SelectionMoleculeByOrderAction::canUndo() {
  return true;
}

bool SelectionMoleculeByOrderAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
